#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Robot@Home cruncher.py """

__author__ = "Gregorio Ambrosio"
__contact__ = "gambrosio[at]uma.es"
__copyright__ = "Copyright 2021, Gregorio Ambrosio"
__date__ = "2021/01/13"
__license__ = "MIT"


import os
import shutil
import sys
import re
import sqlite3
import fire
import cv2


# =========================
#      GLOBAL VARIABLES
# =========================
# Database connection
CON = 0


def copy_rgbd_files(source_folder_path, rgbd_folder_path):

    """ Docstring """

    sql_str_select_rgbd_files = "SELECT * FROM rh2_old2new_rgbd_files;"
    cursor_obj = CON.cursor()
    cursor_obj.execute(sql_str_select_rgbd_files)
    rows = cursor_obj.fetchall()

    for row in rows:
        for i in [2, 3, 4]:
            if row[i]:
                if not os.path.exists(os.path.join(rgbd_folder_path, row[5])):
                    os.makedirs(os.path.join(rgbd_folder_path, row[5]))

                file_ext = re.search(r"\.([^.]+)$", row[i]).group(1)
                if file_ext == 'png':
                    img = cv2.imread(os.path.join(source_folder_path, row[1], row[i]), cv2.IMREAD_COLOR)
                    img_rot = cv2.rotate(img, cv2.ROTATE_90_COUNTERCLOCKWISE)
                    cv2.imwrite(os.path.join(rgbd_folder_path, row[5], row[i+4]), img_rot)
                else:
                    shutil.copy(
                        os.path.join(source_folder_path, row[1], row[i]),
                        os.path.join(rgbd_folder_path, row[5], row[i+4])
                    )
        sys.stdout.write("\rProcessing rgbd row %d " % (rows.index(row)))
        sys.stdout.flush()
    print("\n")


def copy_scene_files(source_folder_path, scene_folder_path):

    """ Docstring """

    sql_str_select_rgbd_files = "SELECT * FROM rh2_old2new_scene_files;"
    cursor_obj = CON.cursor()
    cursor_obj.execute(sql_str_select_rgbd_files)
    rows = cursor_obj.fetchall()

    for row in rows:
        if not os.path.exists(os.path.join(scene_folder_path, row[2])):
            os.makedirs(os.path.join(scene_folder_path, row[2]))
        shutil.copy(
            os.path.join(source_folder_path, row[1]),
            os.path.join(scene_folder_path, row[2], row[3])
        )
        sys.stdout.write("\rProcessing scene row %d " % (rows.index(row)))
        sys.stdout.flush()
    print("\n")


def copy_files(db_name='rh.db', source_folder_path='.', target_folder_path='.'):

    """ Docstring """

    global CON

    # =====================
    #   SQLite initialize
    # =====================

    # global database connection
    CON = sql_connection(db_name)

    # Folder stuff
    rgbd_folder_path = os.path.join(target_folder_path, 'rgbd')
    scene_folder_path = os.path.join(target_folder_path, 'scene')

    if not os.path.exists(target_folder_path):
        os.makedirs(target_folder_path)
    if not os.path.exists(rgbd_folder_path):
        os.makedirs(rgbd_folder_path)
    if not os.path.exists(scene_folder_path):
        os.makedirs(scene_folder_path)

    copy_rgbd_files(source_folder_path, rgbd_folder_path)
    copy_scene_files(source_folder_path, scene_folder_path)

    # =====================
    #  Closing connections
    # =====================
    CON.close()


def sql_connection(database_name):

    """ Docstring """

    global CON

    try:
        CON = sqlite3.connect(database_name)
        print("Connection is established: ", database_name)
        return CON

    except NameError:

        print(NameError)


def main():

    """ Docstring """

    fire.Fire(copy_files)

    # main return
    return 0


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = "Gregorio Ambrosio"
__contact__ = "gambrosio[at]uma.es"
__copyright__ = "Copyright 2020, Gregorio Ambrosio"
__date__ = "2020/10/12"
__license__ = "MIT"

# import robotathome.dataset

from robotathome.dataset import Dataset
import cv2


rhds = Dataset("MyRobot@Home", path=".", autoload=False)
rhds.unit["lblrgbd"].load_data()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Robot@Home Python API """

__author__ = "Gregorio Ambrosio"
__contact__ = "gambrosio[at]uma.es"
__copyright__ = "Copyright 2020, Gregorio Ambrosio"
__date__ = "2020/06/17"
__license__ = "GPLv3"

import cv2
from robotathome.dataset import Dataset


def rgbd():
    """ About rgbd data """

    global rhds
    tab = 4
    print(rhds.unit["rgbd"])
    home_sessions = rhds.unit["rgbd"].home_sessions
    #print(str(home_sessions).expandtabs(0))
    #for home_session in home_sessions:
    #    print(str(home_session.rooms).expandtabs(tab))
    #    for room in home_session.rooms:
    #        print(str(room.name).expandtabs(tab*2))
    print(home_sessions[0].name)
    print(home_sessions[0].rooms[0].name)
    print(home_sessions[0].rooms[0].folder_path)

    path = home_sessions[0].rooms[0].folder_path

    # print(home_sessions[0].rooms[0].sensor_observations)

    sensor = home_sessions[0].rooms[0].sensor_observations[926]

    files = sensor.load_files()
    intensity_file = sensor.get_intensity_file()
    depth_file = sensor.get_depth_file()
    print(files)
    print(intensity_file)
    print(depth_file)

    sensor.show_intensity_image()
    sensor.show_depth_image()
    print('Press a key to continue ...')
    cv2.waitKey(0)
    cv2.destroyAllWindows()


def lblrgbd():
    """ About labelled rgbd data """
    global rhds
    tab = 4
    print(rhds.unit["lblrgbd"])
    home_sessions = rhds.unit["lblrgbd"].home_sessions
    #print(str(home_sessions).expandtabs(0))
    #for home_session in home_sessions:
    #    print(str(home_session.rooms).expandtabs(tab))
    #    for room in home_session.rooms:
    #        print(str(room.name).expandtabs(tab*2))
    print(home_sessions[0].name)
    print(home_sessions[0].rooms[0].name)
    print(home_sessions[0].rooms[0].folder_path)

    path = home_sessions[0].rooms[0].folder_path

    # print(home_sessions[0].rooms[0].sensor_observations)

    sensor = home_sessions[0].rooms[0].sensor_observations[0]

    files = sensor.load_files()
    intensity_file = sensor.get_intensity_file()
    depth_file = sensor.get_depth_file()
    labels_file = sensor.get_labels_file()
    print(files)
    print(intensity_file)
    print(depth_file)
    print(labels_file)

    labels = sensor.get_labels()
    print(labels)
    # print(labels.as_dict_name())
    #print(labels.as_dict_id())
    # mask = sensor.__get_mask()
    #label_mask = sensor.get_label_mask(10)

    sensor.show_intensity_image()
    # sensor.show_depth_image()
    # sensor.show_label_mask_image(10)
    print('Press a key to continue ...')
    cv2.waitKey(0)
    cv2.destroyAllWindows()


def lsrscan():
    """ About laser scans """

    global rhds
    tab = 4
    print(rhds.unit["lsrscan"])
    home_sessions = rhds.unit["lsrscan"].home_sessions

    print(home_sessions[0].name)
    print(home_sessions[0].rooms[0].name)
    print(home_sessions[0].rooms[0].folder_path)
    print(home_sessions[0].rooms[0].sensor_sessions)

    path = home_sessions[0].rooms[0].folder_path

    sensor = home_sessions[0].rooms[0].sensor_sessions[0].sensor_observations[0]
    print(sensor)
    print(type(sensor))
    print(sensor.load_files())
    print(sensor.get_type())
    print(type(sensor))
    laser_scan = sensor.get_laser_scan()
    print(laser_scan)
    print(laser_scan.vector_of_scans)
    print(laser_scan.vector_of_valid_scans)


def raw():

    """ About raw data """
    global rhds
    """
    tab = 4
    print(rhds.unit["raw"])
    home_sessions = rhds.unit["raw"].home_sessions
    """
    """
    print(str(home_sessions).expandtabs(0))
    for home_session in home_sessions:
        print(str(home_session.rooms).expandtabs(tab))
        for room in home_session.rooms:
            print(str(room.name).expandtabs(tab*2))
    """

    print(home_sessions[0].name)
    print(home_sessions[0].rooms[0].name)
    print(home_sessions[0].rooms[0].folder_path)

    path = home_sessions[0].rooms[0].folder_path

    sensor = home_sessions[0].rooms[0].sensor_observations[0]
    print(sensor)
    print(type(sensor))
    print(sensor.load_files())
    print(sensor.get_type())
    print(type(sensor))
    laser_scan = sensor.get_laser_scan(path)
    print(laser_scan)
    #print(laser_scan.vector_of_scans)
    #print(laser_scan.vector_of_valid_scans)

    sensor = home_sessions[0].rooms[0].sensor_observations[24]

    files = sensor.load_files()
    intensity_file = sensor.get_intensity_file()
    depth_file = sensor.get_depth_file()
    print(files)
    print(intensity_file)
    print(depth_file)

    sensor.show_intensity_file()
    sensor.show_depth_file()

    input('Press <enter> to continue')


def chelmnts():
    """  About categories """

    global rhds
    """
    print(rhds.unit["chelmnts"])
    print(rhds.unit["chelmnts"].get_home_names())
    home_dict = rhds.unit["chelmnts"].get_category_home_sessions()
    reversed_home_dict = dict(map(reversed, home_dict.items()))
    print(home_dict)
    print(reversed_home_dict)
    print(rhds.unit["chelmnts"].get_category_rooms())
    print(rhds.unit["chelmnts"].get_category_objects())
    """

    """
    homes = rhds.unit["chelmnts"].home_sessions
    tab = 4
    for home in homes:
        print(str(home).expandtabs(0))
        for room in home.rooms:
            print(str(room).expandtabs(tab))
            for object in room.objects:
                print(str(object).expandtabs(tab*2))
                print((str(object.features).expandtabs(tab*3)))
                print(object.features.as_dict())
            for relation in room.relations:
                print(str(relation).expandtabs(tab*2))
                print(str(relation.features).expandtabs(tab*3))
                print(relation.features.as_dict())
            for observation in room.observations:
                print(str(observation).expandtabs(tab*2))
                print(str(observation.features).expandtabs(tab*3))
                print(observation.features.as_dict())
                print(str(observation.scan_features).expandtabs(tab*3))
                print(observation.scan_features.as_dict())
    """

    """ About Home sessions, room, objects, relations and observations"""
    print('######## HOME #########')
    home_session = rhds.unit["chelmnts"].home_sessions[0]
    print(home_session)
    print(home_session.as_list())
    print(home_session.as_dict())

    print('######## HOMES #########')
    home_sessions = rhds.unit["chelmnts"].home_sessions
    print(home_sessions)
    print(home_sessions.get_ids())
    print(home_sessions.get_names())
    print(home_sessions.as_dict_id())
    print(home_sessions.as_dict_name())

    print('######## HOME.ROOM #########')
    room = home_sessions[0].rooms[0]
    print(room)
    print(room.as_list())
    print(room.as_dict())

    print('######## HOME.ROOMS #########')
    rooms = home_sessions[0].rooms
    print(rooms)
    print(rooms.get_ids())
    print(rooms.get_names())
    print(rooms.as_dict_name())
    print(rooms.as_dict_id())

    print('######## HOME.ROOM.OBJECT #########')
    object = home_sessions[0].rooms[0].objects[0]
    print(object)
    print(object.as_dict())

    print('######## HOME.ROOM.OBJECT.FEATURES #########')
    object_features = home_sessions[0].rooms[0].objects[0].features
    print(object_features)
    print(object_features.as_dict())

    print('######## HOME.ROOM.OBJECTS #########')
    objects = home_sessions[0].rooms[0].objects
    print(objects)
    print(objects.get_ids())
    print(objects.get_names())
    print(objects.as_dict_name())
    print(objects.as_dict_id())

    print('######## HOME.ROOM.RELATION #########')
    relation = home_sessions[0].rooms[0].relations[0]
    print(relation)
    print(relation.as_list())
    print(relation.as_dict())

    print('######## HOMES.ROOMS.RELATION.FEATURES #########')
    relation_features = home_sessions[0].rooms[0].relations[0].features
    print(relation_features)
    print(relation_features.as_dict())

    print('######## HOME.ROOM.RELATIONS #########')
    relations = home_sessions[0].rooms[0].relations
    print(relations)
    print(relations.get_ids())
    print(relations.get_obj1_ids())
    print(relations.get_obj2_ids())
    print(relations.as_dict_id())
    #print(relations.as_dict_obj1_id())
    #print(relations.as_dict_obj2_id())

    print('######## HOME.ROOM.OBSERVATION #########')
    observation = home_sessions[0].rooms[0].observations[0]
    print(observation)
    print(observation.as_list())
    print(observation.as_dict())

    print('######## HOME.ROOM.OBSERVATION.FEATURES #########')
    observation_features = home_sessions[0].rooms[0].observations[0].features
    print(observation_features)
    print(observation_features.as_dict())

    print('######## HOME.ROOM.OBSERVATION.SCAN_FEATURES #########')
    observation_scan_features = home_sessions[0].rooms[0].observations[0].scan_features
    print(observation_scan_features)
    print(observation_scan_features.as_dict())

    print('######## HOME.ROOM.OBSERVATIONS #########')
    observations = home_sessions[0].rooms[0].observations
    print(observations)
    print(observations.get_ids())
    print(observations.get_sensor_names())
    print(observations.as_dict_id())
    print(observations.as_dict_sensor_name())


def geomap():
    """ About 2d geometric maps """
    global rhds
    tab = 4
    print(rhds.unit["2dgeomap"])
    homes = rhds.unit["2dgeomap"].homes
    print(str(homes).expandtabs(0))
    for home in homes:
        print(str(home).expandtabs(0))
        # print(str(home.rooms).expandtabs(tab*1))
        for room in home.rooms:
            print(str(room).expandtabs(tab*1))
            print('        number of points: ' + str(len(room.points)))
            #for point in room.points:
            #    print(str(point).expandtabs(tab*2))
    print(homes[0].rooms[0].points[0])


def hometopo():
    """ About Home Topologies """
    global rhds
    tab = 4
    print(rhds.unit["hometopo"])
    homes = rhds.unit["hometopo"].homes
    print(str(homes).expandtabs(0))
    for home in homes:
        print(str(home.topo_relations).expandtabs(1))
        for topo_relation in home.topo_relations:
            print(str(topo_relation).expandtabs(tab*2))
    print(homes[0])
    print(homes[0].topo_relations[0].room1_name)
    print(homes[0].topo_relations[0].room2_name)
    print(homes[0].topo_relations[0].as_dict())
    print(homes[0].topo_relations.as_dict())
    print(homes[0].as_dict())
    print(homes.as_dict())


def main():
    """
    Multiline comment
    """

    # http://mapir.isa.uma.es/mapirwebsite/index.php/mapir-downloads/203-robot-at-home-dataset.html#downloads
    # /media/goyo/WDGREEN2TB-A/Users/goyo/Documents

    global rhds

    rhds = Dataset("MyRobot@Home")

    """ About data units """
    """
    print(rhds)
    print(rhds["hometopo"].check_integrity(verbose=True))
    print(rhds["hometopo"].hash_for_directory())
    rhds["hometopo"].download()
    """

    rhds["lblscene"].download()


    print("That's all !!!")


if __name__ == "__main__":
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
import os
import sys
import robotathome as rh
# import tests


class Test(unittest.TestCase):
    ''' Test of get_sensor_observation_files '''

    # nittest.skip("testing skipping")
    def setUp(self):
        rh.log.enable_logger(sink=sys.stderr, level="TRACE")
        rh.logger.trace("*** Test.setUp")
        rh.logger.info("""
        Remember:
        python -m unittest <testModule>.<className>.<function_name>
        e.g.
        python -m unittest test_toolbox.Test.test_get_home_names
        """)


        # self.rh_path = '/media/goyo/WDGREEN2TB-A/Users/goyo/Documents/PhD2020/Robot@Home_DataSet_v2.0.0/'
        # self.rgbd_path = os.path.join(self.rh_path, 'files/rgbd')
        # self.db_filename = 'rh.db'
        # self.wspc_path = '/media/goyo/WDGREEN2TB-A/Users/goyo/Documents/PhD2020/WORKSPACE'

        self.rh_path = '/home/goyo/goyo2/Robot@Home_DataSet_v2.0.0'
        self.rgbd_path = os.path.join(self.rh_path, 'files/rgbd')
        self.db_filename = 'rh.db'
        self.wspc_path = '/home/goyo/goyo2/WORKSPACE'

        self.rh_obj = rh.RobotAtHome(self.rh_path, self.wspc_path)

    def tearDown(self):
        rh.logger.trace("*** Test.tearDown")
        del self.rh_obj

    def test_constructor_and_destructor(self):
        """
        Testing of RobotAtHome class constructor
        """
        rh_obj = rh.RobotAtHome(self.rh_path, self.wspc_path)
        con = rh_obj.get_con()
        del rh_obj, con

    def test_get_con(self):
        """
        Testing of RobotAtHome.get_con()
        """
        con = self.rh_obj.get_con()
        rh.logger.debug("con: {}", con)

    def test_select_column(self):
        """
        Testing of RobotAtHome.select_column(column_name, table_name)
        """
        rh.logger.trace("*** Testing of RobotAtHome.get_column_names(column_name, table_name)")
        rh.logger.info("Extracting table names from the database")
        column = self.rh_obj.select_column('tbl_name', 'sqlite_temp_master')
        self.assertEqual(len(column), 1)
        rh.logger.debug("Lenght of column list: {}", len(column))

    def test_get_home_session_names(self):
        """
        Testing of RobotAtHome.get_session_names()
        """
        rh.logger.trace("*** Testing of RobotAtHome.get_home_session_names()")
        rh.logger.info("Extracting home session names from the database")
        column = self.rh_obj.get_home_session_names()
        self.assertEqual(column,['alma-s1', 'anto-s1', 'pare-s1', 'rx2-s1',
                                 'sarmis-s1', 'sarmis-s2', 'sarmis-s3'])
        rh.logger.debug(column)

    def test_get_home_names(self):
        """
        Testing of RobotAtHome.get_home_names()
        """
        rh.logger.trace("*** Testing of RobotAtHome.get_home_names()")
        rh.logger.info("Extracting home names from the database")
        column = self.rh_obj.get_home_names()
        self.assertEqual(column, ['alma', 'anto', 'pare', 'rx2', 'sarmis'])
        rh.logger.debug(column)

    def test_get_room_names(self):
        """
        Testing of RobotAtHome.get_room_names()
        """
        rh.logger.trace("*** Testing of RobotAtHome.get_room_names()")
        rh.logger.info("Extracting room names from the database")
        column = self.rh_obj.get_room_names()
        self.assertEqual(column, ['alma_bathroom1', 'alma_bedroom1',
                                  'alma_fullhouse1', 'alma_kitchen1',
                                  'alma_livingroom1', 'alma_masterroom1',
                                  'anto_bathroom1', 'anto_bathroom2',
                                  'anto_bedroom1', 'anto_bedroom2',
                                  'anto_corridor1', 'anto_dressingroom1',
                                  'anto_fullhouse1', 'anto_kitchen1',
                                  'anto_livingroom1', 'anto_masterroom1',
                                  'pare_bathroom1', 'pare_bathroom2',
                                  'pare_bedroom1', 'pare_bedroom2',
                                  'pare_corridor1', 'pare_fullhouse1',
                                  'pare_hall1', 'pare_kitchen1',
                                  'pare_livingroom1', 'pare_livingroom2',
                                  'pare_masterroom1', 'rx2_bathroom1',
                                  'rx2_bedroom1', 'rx2_fullhouse1',
                                  'rx2_kitchen1', 'rx2_livingroom1',
                                  'sarmis_bathroom1', 'sarmis_bathroom2',
                                  'sarmis_bedroom1', 'sarmis_bedroom2',
                                  'sarmis_bedroom3', 'sarmis_corridor1',
                                  'sarmis_fullhouse1', 'sarmis_kitchen1',
                                  'sarmis_livingroom1']
                         )

        rh.logger.debug(column)

    def test_get_room_type_names(self):
        """
        Testing of RobotAtHome.get_room_type_names()
        """
        rh.logger.trace("*** Testing of RobotAtHome.get_room_type_names()")
        rh.logger.info("Extracting room type names from the database")
        column = self.rh_obj.get_room_type_names()
        self.assertEqual(column, ['bathroom', 'bedroom', 'corridor',
                                  'dressingroom', 'fullhouse', 'hall',
                                  'kitchen', 'livingroom', 'masterroom'])
        rh.logger.debug(column)

    def test_get_sensor_names(self):
        """
        Testing of RobotAtHome.get_sensor_names()
        """
        rh.logger.trace("*** Testing of RobotAtHome.get_sensor_names()")
        rh.logger.info("Extracting sensor names from the database")
        column = self.rh_obj.get_sensor_names()
        self.assertEqual(column, ['HOKUYO1', 'RGBD_1', 'RGBD_2',
                                  'RGBD_3', 'RGBD_4'])
        rh.logger.debug(column)

    def test_get_sensor_type_names(self):
        """
        Testing of RobotAtHome.get_sensor_type_names()
        """
        rh.logger.trace("*** Testing of RobotAtHome.get_sensor_type_names()")
        rh.logger.info("Extracting sensor type names from the database")
        column = self.rh_obj.get_sensor_type_names()
        self.assertEqual(column, ['LASER SCANNER', 'RGBD CAMERA'])
        rh.logger.debug(column)

    def test_get_object_type_names(self):
        """
        Testing of RobotAtHome.get_object_type_names()
        """
        rh.logger.trace("*** Testing of RobotAtHome.get_object_type_names()")
        rh.logger.info("Extracting object type names from the database")
        column = self.rh_obj.get_object_type_names()
        self.assertEqual(len(column), 183)
        rh.logger.debug("Length of object type names list: {}", len(column))

    def test_get_sensor_observation_files(self):
        """
        Testing of get_sensor_observation_files
        """
        rh.logger.trace("*** Testing of RobotAtHome.get_sensor_observation_files()")
        rh.logger.info("Extracting file names from sensor_observations")
        rows = self.rh_obj.get_sensor_observation_files('lblrgbd',
                                                        'anto-s1',
                                                        0,
                                                        'anto_livingroom1',
                                                        'RGBD_2')
        self.assertEqual(len(rows), 355)
        rh.logger.debug("Number of returned rows: {}", len(rows))

    def test_get_video_from_rgbd(self):
        """
        Testing get_video_from_rgbd
        """
        rh.logger.trace("*** Testing of RobotAtHome.get_video_from_rgbd()")
        rh.logger.info("Making a video with images from a room")
        video_file_name = self.rh_obj.get_video_from_rgbd(
            'lblrgbd',
            'anto-s1',
            0,
            'anto_livingroom1',
            'RGBD_2'
        )

        video_file_size = os.path.getsize(
            os.path.join(self.wspc_path, video_file_name)
        )
        self.assertGreater(video_file_size, 6345000)
        rh.logger.debug("Video file name: {}", video_file_name)
        rh.logger.debug("Video file size: {}", video_file_size)

    def test_get_composed_video_from_lblrgbd(self):
        """
        Testing get_composed_video_from_lblrgbd
        """
        rh.logger.trace("*** Testing of RobotAtHome.get_composed_video_from_lblrgbd()")
        rh.logger.info("Making a composed video with 4 images per frame from a room")
        video_file_name = self.rh_obj.get_composed_video_from_lblrgbd(
            'anto-s1',
            0,
            'anto_livingroom1'
        )
        video_file_size = os.path.getsize(
            os.path.join(self.wspc_path, video_file_name)
        )
        self.assertGreater(video_file_size, 22273000)
        rh.logger.debug("Video file name: {}", video_file_name)
        rh.logger.debug("Video file size: {}", video_file_size)

    def test_process_with_yolo(self):
        """
        Testing get_video_from_rgbd
        """
        rh.logger.trace("*** Testing of RobotAtHome.process_with_yolo()")
        rh.logger.info("Processing images from a room with yolo")
        self.rh_obj.process_with_yolo(
            'lblrgbd',
            'anto-s1',
            0,
            'anto_livingroom1',
            'RGBD_2'
        )

    def test_process_with_rcnn(self):
        """
        Testing get_video_from_rgbd
        """
        rh.logger.trace("*** Testing of RobotAtHome.process_with_rcnn()")
        rh.logger.info("Processing images from a room with rcnn")
        my_result = self.rh_obj.process_with_rcnn(
            'lblrgbd',
            'anto-s1',
            0,
            'anto_livingroom1',
            'RGBD_2'
        )


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Robot@Home SQLizer """

__author__ = "Gregorio Ambrosio"
__contact__ = "gambrosio[at]uma.es"
__copyright__ = "Copyright 2021, Gregorio Ambrosio"
__date__ = "2021/01/13"
__license__ = "MIT"


import os.path
import sys
import sqlite3
import re
import fire
from robotathome.dataset import Dataset


# =========================
#      GLOBAL VARIABLES
# =========================
# Database connection
CON = 0
# Dataset
RHDS = None
# Reversed dictionaries
# (to avoid id queries as SELECT id ... WHERE name = ...)
OBJECT_TYPES_DICT_REVERSED = {}
SENSOR_TYPES_DICT = {}
SENSORS_DICT_REVERSED = {}
HOME_SESSIONS_DICT_REVERSED = {}
HOMES_DICT_REVERSED = {}
ROOM_TYPES_DICT_REVERSED = {}
ROOMS_DICT_REVERSED = {}

def dataset2sql(dataset_path='.', database_name='robotathome.db'):

    """ Docstring """

    global CON
    global RHDS

    # ===================
    #     Robot@Home
    # ===================

    # global RHDS
    RHDS = Dataset("MyRobot@Home", path=dataset_path, autoload=False)

    # =====================
    #   SQLite initialize
    # =====================

    # global database connection
    CON = sql_connection(database_name)

    # =====================
    #    Filling tables
    # =====================
    fill_tables()

    # =======================
    #  Executing sql scripts
    # =======================
    add_new_tables_and_views()

    # =====================
    #  Closing connections
    # =====================
    CON.close()


def sql_connection(database_name):

    """ Docstring """

    global CON

    try:
        CON = sqlite3.connect(database_name)
        print("Connection is established: ", database_name)
        return CON

    except NameError:

        print(NameError)


def run_sql_script(sql_file_name):

    """ Docstring """

    try:
        print("Running: ", sql_file_name)
        sql_file = open(sql_file_name)
        sql_as_string = sql_file.read()
        cursor_obj = CON.cursor()
        cursor_obj.executescript(sql_as_string)
    except Exception as e:
        print("Oops! ", sys.exc_info()[0], "occurred while running: ", sql_file_name)
        print('Error on line {}'.format(sys.exc_info()[-1].tb_lineno), type(e).__name__, e)


def create_tables(arg):

    """ Docstring """

    def create_tables_framework():

        """ Docstring """

        run_sql_script('create_tables_framework.sql')

    def create_tables_chelmnts():

        """ Docstring """

        run_sql_script('create_tables_chelmnts.sql')

    def create_tables_raw():

        """ Docstring """

        run_sql_script('create_tables_raw.sql')

    def create_tables_rgbd():

        """ Docstring """

        run_sql_script('create_tables_rgbd.sql')

    def create_tables_lblrgbd():

        """ Docstring """

        run_sql_script('create_tables_lblrgbd.sql')

    def create_tables_lsrscan():

        """ Docstring """

        run_sql_script('create_tables_lsrscan.sql')

    def create_tables_rctrscene():

        """ Docstring """

        run_sql_script('create_tables_rctrscene.sql')

    def create_tables_lblscene():

        """ Docstring """

        run_sql_script('create_tables_lblscene.sql')

    def create_tables_2dgeomap():

        """ Docstring """

        run_sql_script('create_tables_2dgeomap.sql')

    def create_tables_hometopo():

        """ Docstring """

        run_sql_script('create_tables_hometopo.sql')


    switcher = {
        "framework" : create_tables_framework,
        "chelmnts"  : create_tables_chelmnts,
        "raw"       : create_tables_raw,
        "rgbd"      : create_tables_rgbd,
        "lblrgbd"   : create_tables_lblrgbd,
        "lsrscan"   : create_tables_lsrscan,
        "rctrscene" : create_tables_rctrscene,
        "lblscene"  : create_tables_lblscene,
        "2dgeomap"  : create_tables_2dgeomap,
        "hometopo"  : create_tables_hometopo,
    }
    func = switcher.get(arg, lambda: "Invalid argument")
    func()


def fill_tables():

    """ Docstring """

    def fill_tables_framework_data():
        # =============================================================
        #                        FRAMEWORK_DATA
        # =============================================================

        # Set some needed tables with no explicit data
        dataunit_name = "raw"
        create_tables("framework")
        if RHDS.unit[dataunit_name].load_data():
            set_framework_data(dataunit_name)

    def fill_tables_chelmnts():
        # =============================================================
        #                           CHELMNTS
        # =============================================================

        dataunit_name = "chelmnts"
        create_tables(dataunit_name)
        if RHDS.unit[dataunit_name].load_data():
            chelmnts(dataunit_name)

    def fill_tables_raw():
        # =============================================================
        #                           RAW
        # =============================================================

        dataunit_name = "raw"
        create_tables(dataunit_name)
        if RHDS.unit[dataunit_name].load_data():
            sensor_data(dataunit_name)

    def fill_tables_rgbd():
        # =============================================================
        #                           RGBD
        # =============================================================

        dataunit_name = "rgbd"
        create_tables(dataunit_name)
        if RHDS.unit[dataunit_name].load_data():
            sensor_data(dataunit_name)

    def fill_tables_lblrgbd():
        # =============================================================
        #                         LBLRGBD
        # =============================================================

        dataunit_name = "lblrgbd"
        create_tables(dataunit_name)
        if RHDS.unit[dataunit_name].load_data():
            sensor_data(dataunit_name, 100000)

    def fill_tables_lsrscan():
        # =============================================================
        #                         LSRSCAN
        # =============================================================

        dataunit_name = "lsrscan"
        create_tables(dataunit_name)
        if RHDS.unit[dataunit_name].load_data():
            sensor_data(dataunit_name, 200000)

    def fill_tables_rctrscene():
        # =============================================================
        #                           RTRSCENE
        # =============================================================

        dataunit_name = "rctrscene"
        create_tables(dataunit_name)
        if RHDS.unit[dataunit_name].load_data():
            scene_data(dataunit_name)

    def fill_tables_lblscene():
        # =============================================================
        #                           LBLSCENE
        # =============================================================

        dataunit_name = "lblscene"
        create_tables(dataunit_name)
        if RHDS.unit[dataunit_name].load_data():
            scene_data(dataunit_name)

    def fill_tables_twodgeomap():
        # =============================================================
        #                          2DGEOMAP
        # =============================================================

        dataunit_name = "2dgeomap"
        create_tables(dataunit_name)
        if RHDS.unit[dataunit_name].load_data():
            twodgeomap(dataunit_name)

    def fill_tables_hometopo():
        # =============================================================
        #                        HOMETOPO
        # =============================================================

        dataunit_name = "hometopo"
        create_tables(dataunit_name)
        if RHDS.unit[dataunit_name].load_data():
            hometopo(dataunit_name)

    fill_tables_framework_data()
    fill_tables_chelmnts()
    fill_tables_raw()
    fill_tables_rgbd()
    fill_tables_lblrgbd()
    fill_tables_lsrscan()
    fill_tables_rctrscene()
    fill_tables_lblscene()
    fill_tables_twodgeomap()
    fill_tables_hometopo()

    print("Tables successfully populated !")


def set_framework_data(dataunit_name):

    """ Docstring """

    global SENSOR_TYPES_DICT
    global SENSORS_DICT_REVERSED
    global HOME_SESSIONS_DICT_REVERSED
    global HOMES_DICT_REVERSED
    global ROOM_TYPES_DICT_REVERSED
    global ROOMS_DICT_REVERSED

    # Get a cursor to execute SQLite statements
    cursor_obj = CON.cursor()

    # Get the dataunit from global RHDS dataset
    dataunit = RHDS.unit[dataunit_name]

    # ================
    #   Sensor types
    # ================

    SENSOR_TYPES_DICT = {0: "LASER SCANNER", 1: "RGBD CAMERA"}
    cursor_obj.executemany("INSERT INTO rh_sensor_types VALUES(?, ?)",
                           SENSOR_TYPES_DICT.items())

    # ================
    #     Sensors
    # ================

    sensors_list = [[0, 0, "HOKUYO1"],
                    [1, 1, "RGBD_1"],
                    [2, 1, "RGBD_2"],
                    [3, 1, "RGBD_3"],
                    [4, 1, "RGBD_4"]]
    cursor_obj.executemany("INSERT INTO rh_sensors VALUES(?, ?, ?)", sensors_list)
    SENSORS_DICT_REVERSED = dict((x[2], x[0]) for x in sensors_list)

    # ============================================================
    #                        FRAMEWORK
    # ============================================================

    home_sessions = dataunit.home_sessions

    # =======================================
    #               HOMES
    # =======================================
    homes = []
    # get home names list from session names
    for home_session in home_sessions:
        homes.append(home_session.name.split("-s")[0])
    # remove duplicates
    homes = list(dict.fromkeys(homes))
    homes_dict = dict(enumerate(homes, start=0))
    HOMES_DICT_REVERSED = dict(map(reversed, homes_dict.items()))
    # print(HOMES_DICT_REVERSED)
    cursor_obj.executemany("INSERT INTO rh_homes VALUES(?,?)",
                           list(enumerate(homes, start=0)))

    CON.commit()

    # ======================================
    #            HOME_SESSIONS
    # ======================================
    sql_str = ("INSERT INTO rh_home_sessions(id, home_id, name)"
               "VALUES(?, ?, ?)")
    home_session_id = 0
    for home_session in home_sessions:
        home_id = HOMES_DICT_REVERSED[home_session.get_home_name()]
        cursor_obj.execute(sql_str,
                           (home_session_id, home_id, home_session.name)
                           )
        home_session_id += 1
    home_sessions_dict = dict(enumerate(home_sessions.get_names(), start=0))
    HOME_SESSIONS_DICT_REVERSED = dict(map(reversed, home_sessions_dict.items()))

    CON.commit()

    # ======================================
    #               ROOM_TYPES
    # ======================================

    room_types = []
    for home_session in home_sessions:
        for room in home_session.rooms:
            room_types.append(re.split('\d+', room.name)[0])
    room_types = list(dict.fromkeys(room_types))
    room_types_dict = dict(enumerate(room_types, start=0))
    ROOM_TYPES_DICT_REVERSED = dict(map(reversed, room_types_dict.items()))
    cursor_obj.executemany("INSERT INTO rh_room_types VALUES(?,?)",
                           list(enumerate(room_types, start=0)))
    CON.commit()

    # ======================================
    #                ROOMS
    # ======================================

    sql_str = ("INSERT INTO rh_rooms(id, home_id,"
               "                     name, room_type_id)"
               "VALUES(?, ?, ?, ?)")
    room_id = 0
    rooms = []
    for home_session in home_sessions:
        home_id = HOMES_DICT_REVERSED[home_session.get_home_name()]
        home_session_id = HOME_SESSIONS_DICT_REVERSED[home_session.name]
        for room in home_session.rooms:
            room_name = re.split('_\d+', room.name)[0]
            room_type_id = ROOM_TYPES_DICT_REVERSED[re.split('\d+', room.name)[0]]
            if home_session.get_home_name() + "_" + room_name not in rooms:
                cursor_obj.execute(sql_str,
                                   (room_id,
                                    home_id,
                                    home_session.get_home_name() + "_" + room_name,
                                    room_type_id)
                                   )
                rooms.append(home_session.get_home_name() + "_" + room_name)
                room_id += 1
    rooms = list(dict.fromkeys(rooms))
    rooms_dict = dict(enumerate(rooms, start=0))
    ROOMS_DICT_REVERSED = dict(map(reversed, rooms_dict.items()))

    print("\n")

    CON.commit()


def chelmnts(dataunit_name):

    """ Docstring """

    # Get a cursor to execute SQLite statements
    cursor_obj = CON.cursor()

    # Get the dataunit from global RHDS dataset
    dataunit = RHDS.unit[dataunit_name]

    # =============================================================
    #                           CHELMNTS
    # =============================================================

    global OBJECT_TYPES_DICT_REVERSED

    # ================
    #    Object types
    # ================

    object_types_dict = dataunit.get_category_objects()
    cursor_obj.executemany("INSERT INTO rh_object_types VALUES(?,?)",
                           object_types_dict.items())
    CON.commit()

    OBJECT_TYPES_DICT_REVERSED = dict(map(reversed, object_types_dict.items()))
    # print(object_types_dict)
    # print(OBJECT_TYPES_DICT_REVERSED)

    # ===============
    #  Home Sessions
    # ===============

    home_sessions = dataunit.home_sessions
    for home_session in home_sessions:
        home_id = HOMES_DICT_REVERSED[home_session.get_home_name()]
        home_session_id = HOME_SESSIONS_DICT_REVERSED[home_session.name]
        for room in home_session.rooms:
            # print(home_session.get_home_name(), home_session.name, room.name.split('_')[0])
            # print(home_session.get_home_name(), home_session.name, room.name)
            # room_id = ROOMS_DICT_REVERSED[home_session.get_home_name() + "_" + room.name]
            room_id = ROOMS_DICT_REVERSED[home_session.get_home_name() + "_" + re.split('_\d+', room.name)[0]]
            if len(room.name.split('_')) > 1:
                home_subsession_id = int(room.name.split('_')[1])-1
            else:
                home_subsession_id = 0

            # ===============
            #     Objects
            # ===============
            for object in room.objects:
                # print(home_id, home_session_id, home_subsession_id, room_id, object.id, object.name, object.type_id)
                # print(object.features)
                sql_str = "INSERT INTO rh_objects(\
                                                 id, \
                                                 room_id, \
                                                 home_id, \
                                                 home_session_id,\
                                                 home_subsession_id,\
                                                 name, object_type_id, \
                                                 planarity, \
                                                 scatter, \
                                                 linearity, \
                                                 min_height, \
                                                 max_height, \
                                                 centroid_x, \
                                                 centroid_y, \
                                                 centroid_z, \
                                                 volume, \
                                                 biggest_area, \
                                                 orientation, \
                                                 hue_mean, \
                                                 saturation_mean, \
                                                 value_mean, \
                                                 hue_stdv, \
                                                 saturation_stdv, \
                                                 value_stdv, \
                                                 hue_histogram_0, \
                                                 hue_histogram_1, \
                                                 hue_histogram_2, \
                                                 hue_histogram_3, \
                                                 hue_histogram_4, \
                                                 value_histogram_0, \
                                                 value_histogram_1, \
                                                 value_histogram_2, \
                                                 value_histogram_3, \
                                                 value_histogram_4, \
                                                 saturation_histogram_0, \
                                                 saturation_histogram_1, \
                                                 saturation_histogram_2, \
                                                 saturation_histogram_3, \
                                                 saturation_histogram_4  \
                                                 )  \
                                   VALUES(?, ?, ?, ?, ?, ?, ?, \
                                          ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, \
                                          ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, \
                                          ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, \
                                          ?, ?)"
                cursor_obj.execute(sql_str,
                                   ([object.id,
                                     room_id,
                                     home_id,
                                     home_session_id,
                                     home_subsession_id,
                                     object.name,
                                     object.type_id] +
                                     object.features[0:32])
                                   )
                sys.stdout.write("\robject: %d" % (int(object.id)))

            # ===============
            #    Relations
            # ===============
            for relation in room.relations:
                sql_str = ("INSERT INTO rh_relations("
                           "id, "
                           "room_id, "
                           "home_id, "
                           "home_session_id, "
                           "home_subsession_id, "
                           "obj1_id, obj2_id, "
                           "minimum_distance, "
                           "perpendicularity, "
                           "vertical_distance, "
                           "volume_ratio, "
                           "is_on, "
                           "abs_hue_stdv_diff, "
                           "abs_saturation_stdv_diff, "
                           "abs_value_stdv_diff, "
                           "abs_hue_mean_diff, "
                           "abs_saturation_mean_diff, "
                           "abs_value_mean_diff)"
                           "VALUES(?, ?, ?, ?, ?, ?, ?,"
                           "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"
                           )
                cursor_obj.execute(sql_str,
                                   ([relation.id,
                                     room_id,
                                     home_id,
                                     home_session_id,
                                     home_subsession_id,
                                     relation.obj1_id,
                                     relation.obj2_id] +
                                     relation.features[0:11]
                                    )
                                   )
                #sys.stdout.write("\rrelation: %d" % (relation.id))

            # ===============
            #  Observations
            # ===============

            for observation in room.observations:
                sql_str = ("INSERT INTO rh_observations("
                           "id, "
                           "room_id, "
                           "home_id, "
                           "home_session_id, "
                           "home_subsession_id, "
                           "sensor_id, "
                           "mean_hue, "
                           "mean_saturation, "
                           "mean_value, "
                           "hue_stdv, "
                           "saturation_stdv, "
                           "value_stdv, "
                           "hue_histogram_1, "
                           "hue_histogram_2, "
                           "hue_histogram_3, "
                           "hue_histogram_4, "
                           "hue_histogram_5, "
                           "saturation_histogram_1, "
                           "saturation_histogram_2, "
                           "saturation_histogram_3, "
                           "saturation_histogram_4, "
                           "saturation_histogram_5, "
                           "value_histogram_1, "
                           "value_histogram_2, "
                           "value_histogram_3, "
                           "value_histogram_4, "
                           "value_histogram_5, "
                           "distance, "
                           "foot_print, "
                           "volume, "
                           "mean_mean_hue, "
                           "mean_mean_saturation, "
                           "mean_mean_value, "
                           "mean_hue_stdv, "
                           "mean_saturation_stdv, "
                           "mean_value_stdv, "
                           "mean_hue_histogram_1, "
                           "mean_hue_histogram_2, "
                           "mean_hue_histogram_3, "
                           "mean_hue_histogram_4, "
                           "mean_hue_histogram_5, "
                           "mean_saturation_histogram_1, "
                           "mean_saturation_histogram_2, "
                           "mean_saturation_histogram_3, "
                           "mean_saturation_histogram_4, "
                           "mean_saturation_histogram_5, "
                           "mean_value_histogram_1, "
                           "mean_value_histogram_2, "
                           "mean_value_histogram_3, "
                           "mean_value_histogram_4, "
                           "mean_value_histogram_5, "
                           "mean_distance, "
                           "mean_foot_print, "
                           "mean_volume, "
                           "scan_area, "
                           "scan_elongation, "
                           "scan_mean_distance, "
                           "scan_distance_stdv, "
                           "scan_num_of_points, "
                           "scan_compactness, "
                           "scan_compactness2, "
                           "scan_linearity, "
                           "scan_scatter) "
                           "VALUES(?, ?, ?, ?, ?, ?, "
                           "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
                           "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
                           "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
                           "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
                           "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
                           "?, ?, ?, ?, ?, ?, ?) "
                           )
                cursor_obj.execute(sql_str,
                                   ([observation.id,
                                     room_id,
                                     home_id,
                                     home_session.id,
                                     home_subsession_id,
                                     SENSORS_DICT_REVERSED[observation.sensor_name]] +
                                    observation.features +
                                    observation.scan_features)
                                   )
                # print(observation.objects_id)
                # remove repeated objects ids from the original data
                observation.objects_id = list(dict.fromkeys(observation.objects_id))
                objects_in_observation_list = zip(observation.objects_id,
                                  [observation.id]*len(observation.objects_id))
                # print(list(my_whatever))
                sql_str = ("INSERT INTO rh_objects_in_observation("
                           "object_id, "
                           "observation_id) "
                           "VALUES(?, ?) "
                           )
                # Temporarily deactivate to get a faster development !!!!!!!!!
                cursor_obj.executemany(sql_str, objects_in_observation_list)

    print("\n")

    CON.commit()


def test():
    """ Docstring """

    return


def sensor_data(dataunit_name, first_observation_id=0):

    """ Docstring """

    # Get a cursor to execute SQLite statements
    cursor_obj = CON.cursor()

    # Get the dataunit from global RHDS dataset
    dataunit = RHDS.unit[dataunit_name]

    home_sessions = dataunit.home_sessions

    # num_observations = 0
    # num_rooms = 0
    sensor_observation_id = first_observation_id
    label_id = 0
    scan_id = 0

    sql_str_sensor_observation = (
        "INSERT INTO rh_" + dataunit_name + "("
        "id, "
        "room_id, "
        "home_session_id, "
        "home_subsession_id, "
        "home_id, "
        "name, "
        "sensor_id, "
        "sensor_pose_x, "
        "sensor_pose_y, "
        "sensor_pose_z, "
        "sensor_pose_yaw, "
        "sensor_pose_pitch, "
        "sensor_pose_roll, "
        "laser_aperture, "
        "laser_max_range, "
        "laser_num_of_scans, "
        "time_stamp, "
        "sensor_type, "
        "sensor_file_1, "
        "sensor_file_2, "
        "sensor_file_3, "
        "files_path"
        ") "
        "VALUES( ?, ?, ?, ?, ?, ?, "
        "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? )"
        )

    sql_str_labels = (
        "INSERT INTO rh_" + dataunit_name + "_labels " + "("
        "id, "
        "local_id, "
        "name, "
        "sensor_observation_id, "
        "object_type_id"
        ") "
        "VALUES( ?, ?, ?, ?, ? )"
        )

    sql_str_scans = (
        "INSERT INTO rh_" + dataunit_name + "_scans " + "("
        # "id, "
        "shot_id, "
        "scan, "
        "valid_scan, "
        "sensor_observation_id "
        ") "
        "VALUES( ?, ?, ?, ? )"
        )

    for home_session in home_sessions:
        home_id = HOMES_DICT_REVERSED[home_session.get_home_name()]
        home_session_id = HOME_SESSIONS_DICT_REVERSED[home_session.name]

        for room in home_session.rooms:
            try:
                room_name = home_session.get_home_name() + "_" + re.split('_\d+', room.name)[0]
                room_id = ROOMS_DICT_REVERSED[room_name]
                if len(room.name.split('_')) > 1:
                    home_subsession_id = int(room.name.split('_')[1])-1
                else:
                    home_subsession_id = 0
            except KeyError:
                try:
                    room_name = home_session.get_home_name() + "_" + re.split('_\D+', room.name)[0]
                    room_id = ROOMS_DICT_REVERSED[room_name]
                    home_subsession_id = 0
                except KeyError:
                    print("********* KeyError: room " + room_name + " not found", "\n" )
                    continue

            # print(room.name, "|", room_name, "|", room_id, "|", home_session.name, "|", home_session_id, home_subsession_id)

            if dataunit.get_type() == "DatasetUnitLaserScans":
                num_of_sensor_sessions = len(room.sensor_sessions)
                for sensor_session in range(num_of_sensor_sessions):
                    # print("    ", "num_of_sensor_sessions: ", num_of_sensor_sessions, "sensor_session: ", sensor_session)
                    sensor_observations = room.sensor_sessions[sensor_session].sensor_observations
                    for sensor_observation in sensor_observations:
                        # break
                        sensor_observation.load_files()
                        cursor_obj.execute(sql_str_sensor_observation,
                                           (
                                               sensor_observation_id,
                                               room_id,
                                               home_session_id,
                                               sensor_session,
                                               home_id,
                                               sensor_observation.name,
                                               SENSORS_DICT_REVERSED[sensor_observation.name],
                                               sensor_observation.sensor_pose_x,
                                               sensor_observation.sensor_pose_y,
                                               sensor_observation.sensor_pose_z,
                                               sensor_observation.sensor_pose_yaw,
                                               sensor_observation.sensor_pose_pitch,
                                               sensor_observation.sensor_pose_roll,
                                               4.1847,
                                               5.6,
                                               682,
                                               int(sensor_observation.time_stamp),
                                               0 if sensor_observation.get_type() == 'SensorLaserScanner' else 1,
                                               sensor_observation.files[0],
                                               sensor_observation.files[1] if (len(sensor_observation.files) > 1) else '',
                                               sensor_observation.files[2] if (len(sensor_observation.files) > 2) else '',
                                               sensor_observation.rel_path # os.path.relpath(sensor_observation.path)
                                           )
                                           )

                        laser_scan = sensor_observation.get_laser_scan()
                        for shot_id in range(0, len(laser_scan.vector_of_scans)):
                            cursor_obj.execute(sql_str_scans,
                                               (
                                                   # scan_id,
                                                   shot_id,
                                                   laser_scan.vector_of_scans[shot_id],
                                                   laser_scan.vector_of_valid_scans[shot_id],
                                                   sensor_observation_id
                                               )
                                               )
                            scan_id += 1

                        # print(os.path.relpath(sensor_observation.path))
                        # previous_time = int(sensor_observation.time_stamp)
                        sensor_observation_id += 1
                        sys.stdout.write("\rsensor_observation: %d" % (sensor_observation_id))
                    # print("\n")
            else:
                sensor_observations = room.sensor_observations
                for sensor_observation in sensor_observations:
                    # break
                    sensor_observation.load_files()
                    cursor_obj.execute(sql_str_sensor_observation,
                                       (
                                           sensor_observation_id,
                                           room_id,
                                           home_session_id,
                                           home_subsession_id,
                                           home_id,
                                           sensor_observation.name,
                                           SENSORS_DICT_REVERSED[sensor_observation.name],
                                           sensor_observation.sensor_pose_x,
                                           sensor_observation.sensor_pose_y,
                                           sensor_observation.sensor_pose_z,
                                           sensor_observation.sensor_pose_yaw,
                                           sensor_observation.sensor_pose_pitch,
                                           sensor_observation.sensor_pose_roll,
                                           0,
                                           0,
                                           0,
                                           int(sensor_observation.time_stamp),
                                           0 if sensor_observation.get_type() == 'SensorLaserScanner' else 1,
                                           sensor_observation.files[0],
                                           sensor_observation.files[1] if (len(sensor_observation.files) > 1) else '',
                                           sensor_observation.files[2] if (len(sensor_observation.files) > 2) else '',
                                           sensor_observation.rel_path #os.path.relpath(sensor_observation.path)
                                       )
                                       )
                    if len(sensor_observation.files) > 2:
                        labels = sensor_observation.get_labels()
                        for label in labels:
                            try:
                                object_type_id = OBJECT_TYPES_DICT_REVERSED[re.split('_\d+', label.name)[0]]
                            except KeyError:
                                print("********* KeyError: object " + label.name + " not found", "\n" )
                                object_type_id = -1

                            cursor_obj.execute(sql_str_labels,
                                               (
                                                   label_id,
                                                   label.id,
                                                   label.name,
                                                   sensor_observation_id,
                                                   object_type_id
                                               )
                                               )
                            label_id += 1

                    if sensor_observation.get_type() == "SensorLaserScanner":
                        laser_scan = sensor_observation.get_laser_scan()
                        for shot_id in range(0, len(laser_scan.vector_of_scans)):
                            cursor_obj.execute(sql_str_scans,
                                               (
                                                   # scan_id,
                                                   shot_id,
                                                   laser_scan.vector_of_scans[shot_id],
                                                   laser_scan.vector_of_valid_scans[shot_id],
                                                   sensor_observation_id,
                                               )
                                               )
                            scan_id += 1

                    # print(os.path.relpath(sensor_observation.path))
                    # previous_time = int(sensor_observation.time_stamp)
                    sensor_observation_id += 1
                    sys.stdout.write("\rsensor_observation: %d" % (sensor_observation_id))
                # print("\n")
    print("\n")

    CON.commit()


def scene_data(dataunit_name):

    """ Docstring """

    # Get a cursor to execute SQLite statements
    cursor_obj = CON.cursor()

    # Get the dataunit from global RHDS dataset
    dataunit = RHDS.unit[dataunit_name]

    # =============================================================
    #                         SCENE_DATA
    # =============================================================

    scene_id = 0
    bb_id = 0

    sql_str_scene = (
        "INSERT INTO rh_" + dataunit_name + "("
        "id, "
        "room_id, "
        "home_session_id, "
        "home_subsession_id, "
        "home_id, "
        "scene_file"
        ") "
        "VALUES( ?, ?, ?, ?, ?, ?)"
        )

    sql_str_bb = (
        "INSERT INTO rh_" + dataunit_name + "_bboxes" + "("
        "id, "
        "local_id,"
        "scene_id, "
        "object_id, "
        "object_name, "
        "bb_pose_x, "
        "bb_pose_y, "
        "bb_pose_z, "
        "bb_pose_yaw, "
        "bb_pose_pitch, "
        "bb_pose_roll, "
        "bb_corner1_x, "
        "bb_corner1_y, "
        "bb_corner1_z, "
        "bb_corner2_x, "
        "bb_corner2_y, "
        "bb_corner2_z"
        ") "
        "VALUES( ?, ?, ?, ?, ?, "
        "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? )"
        )

    sql_str_select = (
        "SELECT id FROM rh_objects WHERE "
        "room_id = ? AND "
        "home_id = ? AND "
        "home_session_id = ? AND "
        "home_subsession_id = ? AND "
        "name = ?"
    )


    home_sessions = dataunit.home_sessions
    for home_session in home_sessions:
        home_id = HOMES_DICT_REVERSED[home_session.get_home_name()]
        home_session_id = HOME_SESSIONS_DICT_REVERSED[home_session.name]
        for room in home_session.rooms:
            room_id = ROOMS_DICT_REVERSED[home_session.get_home_name() + "_" + re.split('_\d+', room.name)[0]]
            if len(room.name.split('_')) > 1:
                home_subsession_id = int(room.name.split('_')[1])-1
            else:
                home_subsession_id = 0

            cursor_obj.execute(sql_str_scene,
                               (
                                   scene_id,
                                   room_id,
                                   home_session_id,
                                   home_subsession_id,
                                   home_id,
                                   room.scene_file
                                )
                               )
            # sys.stdout.write("\rscene: %d" % (scene_id))

            # ================================
            #     Bounding boxes (Objects)
            # ================================
            for bb in room.boundingboxes:
                cursor_obj.execute(sql_str_select,
                                   (
                                       room_id,
                                       home_id,
                                       home_session_id,
                                       home_subsession_id,
                                       bb.name
                                   )
                                   )
                row = cursor_obj.fetchone()
                if row is None:
                    object_id = None
                else:
                    object_id = row[0]
                cursor_obj.execute(sql_str_bb,
                                   (
                                       bb_id,
                                       bb.id,
                                       scene_id,
                                       object_id,
                                       bb.name,
                                       bb.bb_pose[0],
                                       bb.bb_pose[1],
                                       bb.bb_pose[2],
                                       bb.bb_pose[3],
                                       bb.bb_pose[4],
                                       bb.bb_pose[5],
                                       bb.bb_corner[0],
                                       bb.bb_corner[1],
                                       bb.bb_corner[2],
                                       bb.bb_corner[3],
                                       bb.bb_corner[4],
                                       bb.bb_corner[5],
                                    )
                                   )
                # sys.stdout.write("\rbounding box: %d" % (bb_id))
                bb_id += 1
            scene_id += 1


    if bb_id > 0:
        print("scenes: %d, bounding boxes: %d" % (scene_id, bb_id))
    else:
        print("scenes: %d" % (scene_id))

    print("\n")

    CON.commit()


def twodgeomap(dataunit_name):

    """ Docstring """

    # Get a cursor to execute SQLite statements
    cursor_obj = CON.cursor()

    # Get the dataunit from global RHDS dataset
    dataunit = RHDS.unit[dataunit_name]

    # =============================================================
    #                          2DGEOMAP
    # =============================================================

    point_id = 0

    sql_str_2dgeomap = (
        "INSERT INTO rh_twodgeomap("
        "id, "
        "home_id, "
        "room_id, "
        "x, "
        "y, "
        "z "
        ") "
        "VALUES( ?, ?, ?, ?, ?, ?)"
        )

    homes = dataunit.homes
    for home in homes:
        home_id = HOMES_DICT_REVERSED[home.name]
        for room in home.rooms:
            try:
                room_id = ROOMS_DICT_REVERSED[home.name+"_"+room.name.split("_")[0]]
            except KeyError:
                continue
            for point in room.points:
                cursor_obj.execute(sql_str_2dgeomap,
                                   (
                                       point_id,
                                       home_id,
                                       room_id,
                                       point.x,
                                       point.y,
                                       point.z
                                    )
                                   )
                sys.stdout.write("\rpoint: %d" % (point_id))
                point_id += 1

    print("\n")

    CON.commit()


def hometopo(dataunit_name):

    """ Docstring """

    # Get a cursor to execute SQLite statements
    cursor_obj = CON.cursor()

    # Get the dataunit from global RHDS dataset
    dataunit = RHDS.unit[dataunit_name]


    # =============================================================
    #                          2DGEOMAP
    # =============================================================

    topo_id = 0

    sql_str_hometopo = (
        "INSERT INTO rh_hometopo("
        "id, "
        "home_id, "
        "room1_id, "
        "room2_id  "
        ") "
        "VALUES( ?, ?, ?, ?)"
        )

    homes = dataunit.homes
    for home in homes:
        home_id = HOMES_DICT_REVERSED[home.name]
        for topo_relation in home.topo_relations:
            room1_id = ROOMS_DICT_REVERSED[home.name+"_"+topo_relation.room1_name]
            room2_id = ROOMS_DICT_REVERSED[home.name+"_"+topo_relation.room2_name]
            cursor_obj.execute(sql_str_hometopo,
                               (
                                   topo_id,
                                   home_id,
                                   room1_id,
                                   room2_id
                                )
                               )
            sys.stdout.write("\rpoint: %d" % (topo_id))
            topo_id += 1

    print("\n")

    CON.commit()


def add_new_tables_and_views():

    """ Docstring """

    sql_file_names = ["create_view_sensor_observations.sql",
                      "create_view_scene_bb_objects.sql",
                      "create_new_rgbd_file_names.sql",
                      "create_new_scene_file_names.sql"]

    for sql_file_name in sql_file_names:
        run_sql_script(sql_file_name)


def main():

    """ Docstring """

    fire.Fire(dataset2sql)

    # try:
    #     fire.Fire(dataset2sql)
    # except Exception as e:
    #     print("Oops! ", sys.exc_info()[0], " occurred.")
    #     print('Error on line {}'.format(sys.exc_info()[-1].tb_lineno), type(e).__name__, e)


    # main return
    return 0


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = "Gregorio Ambrosio"
__contact__ = "gambrosio[at]uma.es"
__copyright__ = "Copyright 2021, Gregorio Ambrosio"
__date__ = "2021/02/12"
__license__ = "MIT"

import sys
import time
import re
import datetime
import sqlite3
import os
import cv2
import pandas as pd
from mxnet import image
from gluoncv import model_zoo, data, utils
from matplotlib import pyplot as plt
import robotathome as rh
# import fire


def time_win2unixepoch(time_stamp):
    ''' Doctring '''
    seconds = time_stamp / 10000000
    epoch = seconds - 11644473600
    datetime_ = datetime.datetime(2000, 1, 1, 0, 0, 0)
    return datetime_.fromtimestamp(epoch)


def time_unixepoch2win(date):
    ''' Doctring '''
    match = re.compile(r'^(\d{4})-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)$').match(date)
    if match:
        datetime_ = datetime.datetime(*map(int, match.groups()))
        windows_timestamp = (time.mktime(datetime_.timetuple()) + 11644473600) * 10000000
    else:
        print("Invalid date format specified: " + date)
        print("Specify a date and time string in the format: \"yyyy-MM-ddTHH:mm:ss\"")
        windows_timestamp = 0
    return windows_timestamp


def tup2list(tup):
    ''' Convert list of tuples into list '''
    return list(sum(tup, ()))


class RobotAtHome():
    """
    RobotAtHome class with methods to manage the Robot@Home dataset v2.x.y
    """

    def __init__(self,
                 rh_path='.',
                 wspc_path='.',
                 db_filename='rh.db',
                 rgbd_path='files/rgbd',
                 scene_path='files/scene'):
        """ RobotAtHome constructor method """
        self.__rh_path = rh_path
        self.__wspc_path = wspc_path
        self.__db_filename = db_filename
        self.__rgbd_path = rgbd_path
        self.__scene_path = scene_path
        self.__con = None
        self.__rgbd_views = []

        # Initialization functions
        self.__open_dataset()
        self.__create_temp_views()

    def __del__(self):
        """ Robot@Home destructor method"""

    def __open_dataset(self):

        """
        This function makes the connection with the database and calls the
        initialization functions, e.g. create temporal views
        """

        db_full_path = os.path.join(self.__rh_path, self.__db_filename)
        rh.logger.debug("db_full_path: {}", db_full_path)

        try:
            self.__con = sqlite3.connect(db_full_path)
            rh.logger.info("Connection is established: {}", self.__db_filename)
        except NameError:
            rh.logger.error("Error while trying to open database: {}", NameError)

    def __close_dataset(self):
        """
        This function closes the connection with the database
        """
        self.__con.close()
        rh.logger.info("The connection with the database has been successfully closed")

    def __create_temp_views(self):
        """
        This function creates temporary views to work on the class environment
        """

        sql_str = '''
        begin transaction;
        drop view if exists rh_temp_lblrgbd;
        create temp view rh_temp_lblrgbd as
        select
            rh_lblrgbd.id,
            rh_lblrgbd.home_session_id as hs_id,
            rh_home_sessions.name as hs_name,
            rh_lblrgbd.home_subsession_id as hss_id,
            rh_lblrgbd.home_id as h_id,
            rh_homes.name as h_name,
            rh_lblrgbd.room_id as r_id,
            rh_rooms.name as r_name,
            rh_lblrgbd.sensor_id as s_id,
            rh_sensors.name as s_name,
            rh_lblrgbd.time_stamp as t,
            rh_lblrgbd.sensor_pose_x as s_px,
            rh_lblrgbd.sensor_pose_y as s_py,
            rh_lblrgbd.sensor_pose_z as s_pz,
            rh_lblrgbd.sensor_pose_yaw as s_pya,
            rh_lblrgbd.sensor_pose_pitch as s_ppi,
            rh_lblrgbd.sensor_pose_roll as s_pro,
            rh2_old2new_rgbd_files.new_file_1 as f1,
            rh2_old2new_rgbd_files.new_file_2 as f2,
            rh2_old2new_rgbd_files.new_file_3 as f3,
            rh2_old2new_rgbd_files.new_path as pth
        from rh_lblrgbd
        inner join rh_home_sessions on home_session_id = rh_home_sessions.id
        inner join rh_homes on rh_lblrgbd.home_id = rh_homes.id
        inner join rh_rooms on rh_lblrgbd.room_id = rh_rooms.id
        inner join rh_sensors on rh_lblrgbd.sensor_id = rh_sensors.id
        inner join rh2_old2new_rgbd_files on rh2_old2new_rgbd_files.id = rh_lblrgbd.id;
        commit;
        '''

        # Get a cursor to execute SQLite statements
        cur = self.__con.cursor()
        cur.executescript(sql_str)

        self.__rgbd_views.append("rh_temp_lblrgbd")
        rh.logger.trace("The view rh_temp_lblrgbd has been created")

    def get_con(self):
        """
        This function returns the sql connection variable
        """
        return self.__con

    def select_column(self, column_name, table_name):
        ''' Return a list column names '''

        # Get a cursor to execute SQLite statements
        cur = self.__con.cursor()

        # Build the query
        sql_str = ("select " + column_name + " from " + table_name + " group by " + column_name + ";")
        cur.execute(sql_str)
        return tup2list(cur.fetchall())

    def __get_temp_sql_object_names(self):
        ''' Return a list with temporary/internal created views'''
        return self.select_column('tbl_name', 'sqlite_temp_master')

    def get_rh2_rgbd_views(self):
        """
        Return a list with RGB-D views
        """
        # There is only one view for now
        return self.__rgbd_views

    def get_home_session_names(self):
        """
        Return a list with home session names
        """
        return self.select_column('name', 'rh_home_sessions')

    def get_home_names(self):
        """
        Return a list with home names '''
        """
        return self.select_column('name', 'rh_homes')

    def get_room_names(self):
        """
        Return a list with room names
        """
        return self.select_column('name', 'rh_rooms')

    def get_room_type_names(self):
        """
        Return a list with room type names
        """
        return self.select_column('name', 'rh_room_types')

    def get_sensor_names(self):
        """
        Return a list with sensor names
        """
        return self.select_column('name', 'rh_sensors')

    def get_sensor_type_names(self):
        """
        Return a list with sensor type names
        """
        return self.select_column('name', 'rh_sensor_types')

    def get_object_type_names(self):
        """
        Return a list with room type names
        """
        return self.select_column('name', 'rh_object_types')

    def get_sensor_observation_files(self,
            source='lblrgbd',
            home_session_name='alma-s1',
            home_subsession=0,
            room_name='alma_masterroom1',
            sensor_name='RGBD_1'
    ):

        """
        This functions queries the database to extract sensor observation
        files filtered by home_session_name, home_subsession, room_name, and
        sensor_name.
        """

        # Get a cursor to execute SQLite statements
        cur = self.__con.cursor()

        switcher = {
            # rh_temp_lblrgbd created in _create_temp_views at the begining
            "lblrgbd": "rh_temp_lblrgbd",
        }
        sensor_observation_table = switcher.get(source, lambda: "Invalid argument")

        # Build the query
        sql_str = (
            '''
            select id, t, pth, f1, f2, f3
            from
            ''' +
            sensor_observation_table +
            '''
            where
                hs_name = ? and
                hss_id = ? and
                r_name = ? and
                s_name = ?
            order by t
            '''
        )

        parms = (home_session_name,
                 home_subsession,
                 room_name,
                 sensor_name)
        cur.execute(sql_str, parms)
        rows = cur.fetchall()

        return rows

    def get_video_from_rgbd(self,
                            source='lblrgbd',
                            home_session_name='alma-s1',
                            home_subsession=0,
                            room_name='alma_masterroom1',
                            sensor_name='RGBD_1',
                            video_file_name=None
                            ):

        """
        This functions ...
        """

        rows = self.get_sensor_observation_files(source,
                                                 home_session_name,
                                                 home_subsession,
                                                 room_name,
                                                 sensor_name)

        # Computing frames per second
        num_of_frames = len(rows)
        seconds = (rows[-1][1] - rows[0][1]) / 10**7
        frames_per_second = num_of_frames / seconds
        rh.logger.debug("frames per second: {:.2f}", frames_per_second)

        # Get frame size
        image_path = rows[0][2]
        file_name = rows[0][4]
        image_path_file_name = os.path.join(self.__rh_path,
                                            self.__rgbd_path,
                                            image_path,
                                            file_name)
        img = cv2.imread(image_path_file_name, cv2.IMREAD_COLOR)
        img_h, img_w, _ = img.shape

        # Opening video file
        if video_file_name is None:
            video_file_name = ''.join(
                [
                    home_session_name,
                    '_', str(home_subsession),
                    '_', room_name,
                    '_', sensor_name,
                    '.avi'
                ]
            )
        fourcc = cv2.VideoWriter_fourcc(*'MJPG')
        video_path_file_name = os.path.abspath(os.path.join(self.__wspc_path,
                                                            video_file_name)
                                               )
        out = cv2.VideoWriter(video_path_file_name,
                              fourcc,
                              frames_per_second,
                              (img_w, img_h))

        for row in rows:
            image_path = row[2]
            file_name = row[4]
            image_path_file_name = os.path.join(self.__rh_path,
                                                self.__rgbd_path,
                                                image_path,
                                                file_name)
            img = cv2.imread(image_path_file_name, cv2.IMREAD_COLOR)
            # cv2.imshow('img', img)
            # if cv2.waitKey(1) & 0xFF == ord('q'):
            #     break
            out.write(img)

        out.release()

        return video_file_name

    def get_composed_video_from_lblrgbd(self,
                                        home_session_name='alma-s1',
                                        home_subsession=0,
                                        room_name='alma_masterroom1',
                                        video_file_name=None
                                        ):

        ''' Docstring '''

        rows_rgbd_1 = self.get_sensor_observation_files('lblrgbd',
                                                        home_session_name,
                                                        home_subsession,
                                                        room_name,
                                                        'RGBD_1')

        rows_rgbd_2 = self.get_sensor_observation_files('lblrgbd',
                                                        home_session_name,
                                                        home_subsession,
                                                        room_name,
                                                        'RGBD_2')

        rows_rgbd_3 = self.get_sensor_observation_files('lblrgbd',
                                                        home_session_name,
                                                        home_subsession,
                                                        room_name,
                                                        'RGBD_3')

        rows_rgbd_4 = self.get_sensor_observation_files('lblrgbd',
                                                        home_session_name,
                                                        home_subsession,
                                                        room_name,
                                                        'RGBD_4')

        # Computing frames per second
        num_of_frames = len(rows_rgbd_1)
        seconds = (rows_rgbd_1[-1][1] - rows_rgbd_1[0][1]) / 10**7
        frames_per_second = num_of_frames / seconds
        rh.logger.debug("frames per second: {:.2f}", frames_per_second)

        # Get frame size
        image_path = rows_rgbd_1[0][2]
        file_name = rows_rgbd_1[0][4]
        image_path_file_name = os.path.join(self.__rh_path,
                                            self.__rgbd_path,
                                            image_path,
                                            file_name)
        img = cv2.imread(image_path_file_name, cv2.IMREAD_COLOR)
        img_h, img_w, _ = img.shape

        # Opening video file
        if video_file_name is None:
            video_file_name = ''.join(
                [
                    home_session_name,
                    '_', str(home_subsession),
                    '_', room_name,
                    '_RGBD_3412',
                    '.avi'
                ]
            )

        fourcc = cv2.VideoWriter_fourcc(*'MJPG')
        video_path_file_name = os.path.abspath(os.path.join(self.__wspc_path,
                                                            video_file_name))
        out = cv2.VideoWriter(video_path_file_name,
                              fourcc,
                              frames_per_second,
                              (4 * img_w, img_h))

        for i, _ in enumerate(rows_rgbd_1):
            image_rgbd_1_path = rows_rgbd_1[i][2]
            file_rgbd_1_name = rows_rgbd_1[i][4]
            image_rgbd_1_path_file_name = os.path.join(self.__rh_path,
                                                       self.__rgbd_path,
                                                       image_rgbd_1_path,
                                                       file_rgbd_1_name)

            image_rgbd_2_path = rows_rgbd_2[i][2]
            file_rgbd_2_name = rows_rgbd_2[i][4]
            image_rgbd_2_path_file_name = os.path.join(self.__rh_path,
                                                       self.__rgbd_path,
                                                       image_rgbd_2_path,
                                                       file_rgbd_2_name)

            image_rgbd_3_path = rows_rgbd_3[i][2]
            file_rgbd_3_name = rows_rgbd_3[i][4]
            image_rgbd_3_path_file_name = os.path.join(self.__rh_path,
                                                       self.__rgbd_path,
                                                       image_rgbd_3_path,
                                                       file_rgbd_3_name)

            image_rgbd_4_path = rows_rgbd_4[i][2]
            file_rgbd_4_name = rows_rgbd_4[i][4]
            image_rgbd_4_path_file_name = os.path.join(self.__rh_path,
                                                       self.__rgbd_path,
                                                       image_rgbd_4_path,
                                                       file_rgbd_4_name)


            img_rgbd_1 = cv2.imread(image_rgbd_1_path_file_name, cv2.IMREAD_COLOR)
            img_rgbd_2 = cv2.imread(image_rgbd_2_path_file_name, cv2.IMREAD_COLOR)
            img_rgbd_3 = cv2.imread(image_rgbd_3_path_file_name, cv2.IMREAD_COLOR)
            img_rgbd_4 = cv2.imread(image_rgbd_4_path_file_name, cv2.IMREAD_COLOR)

            img = cv2.hconcat([img_rgbd_3, img_rgbd_4, img_rgbd_1, img_rgbd_2])

            # cv2.imshow('img', img)
            # if cv2.waitKey(1) & 0xFF == ord('q'):
            #     break

            out.write(img)
        out.release()
        # cv2.destroyAllWindows()
        # return [rows_rgbd_1, rows_rgbd_2, rows_rgbd_3, rows_rgbd_4]
        return video_file_name

    def process_with_yolo(self,
                          source='lblrgbd',
                          home_session_name='alma-s1',
                          home_subsession=0,
                          room_name='alma_masterroom1',
                          sensor_name='RGBD_1',
                          video_file_name=None
                          ):

        """
        This functions ...
        """

        rows = self.get_sensor_observation_files(source,
                                                 home_session_name,
                                                 home_subsession,
                                                 room_name,
                                                 sensor_name)

        # Computing frames per second
        num_of_frames = len(rows)
        seconds = (rows[-1][1] - rows[0][1]) / 10**7
        frames_per_second = num_of_frames / seconds
        rh.logger.debug("frames per second: {:.2f}", frames_per_second)

        # Get frame size
        image_path = rows[0][2]
        file_name = rows[0][4]
        image_path_file_name = os.path.join(self.__rh_path,
                                            self.__rgbd_path,
                                            image_path,
                                            file_name)
        img = cv2.imread(image_path_file_name, cv2.IMREAD_COLOR)
        img_h, img_w, _ = img.shape

        # Opening video file
        if video_file_name is None:
            video_file_name = ''.join(
                [
                    home_session_name,
                    '_', str(home_subsession),
                    '_', room_name,
                    '_', sensor_name,
                    '_by_YOLO',
                    '.avi'
                ]
            )
        fourcc = cv2.VideoWriter_fourcc(*'MJPG')
        video_path_file_name = os.path.abspath(os.path.join(self.__wspc_path,
                                                            video_file_name)
                                               )
        out = cv2.VideoWriter(video_path_file_name,
                              fourcc,
                              frames_per_second,
                              (img_w, img_h))



        ##############################################
        #                     NN
        ##############################################


        #  get NN model
        net = model_zoo.get_model('yolo3_darknet53_coco', pretrained=True)

        nn_out = []
        i = 0
        for row in rows:
            image_path = row[2]
            file_name = row[4]
            image_path_file_name = os.path.join(self.__rh_path,
                                                self.__rgbd_path,
                                                image_path,
                                                file_name)
            i += 1
            sys.stdout.write("\rProcessing frame %i of %i" % (i, len(rows)))
            sys.stdout.flush()

            # core
            try:
                img = image.imread(image_path_file_name)
            except:
                print('%s is not a valid raster image' % image_path_file_name)

            # long_edge_size = img.shape[0]
            short_edge_size = img.shape[1]

            x, img = data.transforms.presets.yolo.load_test(image_path_file_name,
                                                            short=short_edge_size)
            # rh.logger.debug('Shape of pre-processed image: {}', x.shape)
            class_ids, scores, bounding_boxs = net(x)
            nn_out.append([class_ids, scores, bounding_boxs])
            utils.viz.cv_plot_bbox(img,
                                   bounding_boxs[0],
                                   scores[0],
                                   class_ids[0],
                                   class_names=net.classes,
                                   thresh=0.2,
                                   linewidth=1)

            out.write(img)

        out.release()

        return pd.DataFrame(nn_out), video_file_name

    def process_with_rcnn(self,
                          source='lblrgbd',
                          home_session_name='alma-s1',
                          home_subsession=0,
                          room_name='alma_masterroom1',
                          sensor_name='RGBD_1',
                          video_file_name=None
                          ):

        """
        This functions ...
        """

        rows = self.get_sensor_observation_files(source,
                                                 home_session_name,
                                                 home_subsession,
                                                 room_name,
                                                 sensor_name)

        # Computing frames per second
        num_of_frames = len(rows)
        seconds = (rows[-1][1] - rows[0][1]) / 10**7
        frames_per_second = num_of_frames / seconds
        rh.logger.debug("frames per second: {:.2f}", frames_per_second)

        # Get frame size
        image_path = rows[0][2]
        file_name = rows[0][4]
        image_path_file_name = os.path.join(self.__rh_path,
                                            self.__rgbd_path,
                                            image_path,
                                            file_name)
        img = cv2.imread(image_path_file_name, cv2.IMREAD_COLOR)
        img_h, img_w, _ = img.shape

        # Opening video file
        if video_file_name is None:
            video_file_name = ''.join(
                [
                    home_session_name,
                    '_', str(home_subsession),
                    '_', room_name,
                    '_', sensor_name,
                    '_by_RCNN',
                    '.avi'
                ]
            )
        fourcc = cv2.VideoWriter_fourcc(*'MJPG')
        video_path_file_name = os.path.abspath(os.path.join(self.__wspc_path,
                                                            video_file_name)
                                               )
        out = cv2.VideoWriter(video_path_file_name,
                              fourcc,
                              frames_per_second,
                              (img_w, img_h))



        ##############################################
        #                     NN
        ##############################################


        #  get NN model
        net = model_zoo.get_model('yolo3_darknet53_coco', pretrained=True)

        nn_out = []
        i = 0
        for row in rows:
            image_path = row[2]
            file_name = row[4]
            image_path_file_name = os.path.join(self.__rh_path,
                                                self.__rgbd_path,
                                                image_path,
                                                file_name)
            i += 1
            sys.stdout.write("\rProcessing frame %i of %i" % (i, len(rows)))
            sys.stdout.flush()

            # core
            try:
                img = image.imread(image_path_file_name)
            except:
                print('%s is not a valid raster image' % image_path_file_name)

            # long_edge_size = img.shape[0]
            short_edge_size = img.shape[1]

            x, img = data.transforms.presets.rcnn.load_test(image_path_file_name,
                                                            short=short_edge_size)
            # rh.logger.debug('Shape of pre-processed image: {}', x.shape)
            class_ids, scores, bounding_boxs = net(x)
            nn_out.append([class_ids, scores, bounding_boxs])
            utils.viz.cv_plot_bbox(img,
                                   bounding_boxs[0],
                                   scores[0],
                                   class_ids[0],
                                   class_names=net.classes,
                                   thresh=0.2,
                                   linewidth=1)
            out.write(img)

        out.release()

        return pd.DataFrame(nn_out), video_file_name


def main():

    """ Docstring """

    # fire.Fire()

    # try:
    #     fire.Fire(<function>)
    # except Exception as e:
    #     print("Oops! ", sys.exc_info()[0], " occurred.")
    #     print('Error on line {}'.format(sys.exc_info()[-1].tb_lineno), type(e).__name__, e)

    # rh.enable_logger(sink=sys.stderr, level="TRACE")
    rh.logger.trace("Running main() from toolbox.py")

    # print(rh.version.get_version_str())

    # main return
    return 0


if __name__ == "__main__":
    main()
